/*** Syntakticky analyzator jazyka Java-- (JMM) ***/

%start Subor
/* definicia term. symbolov */
%token BOOLEAN  DOUBLE  INT  LONG  VOID
%token TRIEDA
%token CLASS  EXTENDS  CONSTRUCTOR IMPLEMENTS IMPORT
%token PUBLIC PRIVATE PROTECTED
%token STATIC FINAL ABSTRACT NATIVE
%token NEW THIS SUPER RETURN IF ELSE WHILE
%token ID
%token STR_LITERAL
%token INT_LITERAL
%token DBL_LITERAL
%token NULL_LITERAL
%token TRUE_LITERAL
%token FALSE_LITERAL
%token LZZAT  PZZAT  LOZAT  POZAT  LHZAT  PHZAT
%token CIARKA  BCIARKA  BODKA  ROVNE

/* priradenie priorit a asociativnosti operatorom */
%left       AND OR
%nonassoc   LT LE GT GE EQ NE
%left       PLUS MINUS
%left       KRAT DEL
%left       UNARNY

%{
/* prototypy */
int yyparse(void), yylex(void);
void yyerror(char*), xxerror(char*, char*);

/* vlozenie lexikalneho analyzatora */
#include "lex.yy.c"

%}

%%
Subor   : Importy Trieda
        | Subor  Trieda
        ;
Importy : /* e */
        | Importy  Import
        ;
Import  : IMPORT  ID  NATIVE  STR_LITERAL
        ;
Trieda  : CLASS  ModTriedy ID  NadTrieda  ImpRozhrania
          LZZAT  Cleny  PZZAT
        ;
ModTriedy : /* e */
        | PUBLIC
        | ABSTRACT
        | FINAL
        ;
NadTrieda : /* e */
        | EXTENDS  TRIEDA
        ;
ImpRozhrania : /* e */
        | ImpRozhrania IMPLEMENTS TRIEDA
        ;
Cleny   : /* e */
        | Cleny  DatovyClen
        | Cleny  Metoda
        ;
DatovyClen : ModClena TypDat  ID  BCIARKA
        ;
ModClena : Mod1 Mod2
        ;
Mod1    : /* e */
        | PUBLIC
        | PROTECTED
        | PRIVATE
        ;
Mod2    : /* e */
        | STATIC
        | ABSTRACT
        | FINAL
        | STATIC FINAL
        ;
Metoda  : ModClena TypDat  ID  LOZAT  NepParametre  POZAT  TeloMet
        | ModClena TypVoid  ID  LOZAT  NepParametre  POZAT  TeloMet
        | CONSTRUCTOR  LOZAT NepParametre POZAT  TeloMet
        ;
NepParametre : /* e */
        | Parametre
        ;
Parametre : TypDat  ID
        | Parametre  CIARKA  TypDat  ID
        ;
TypVoid : VOID
        ;
TypDat  : INT
        | BOOLEAN
        | DOUBLE
        | TRIEDA
        | TRIEDA LHZAT PHZAT
        ;
TeloMet : BCIARKA
        | LZZAT  Prikazy  PZZAT
        ;
Prikazy : /* e */
        | Prikazy Prikaz
        | Prikazy error
        ;
Prikaz  : BCIARKA
        | TypDat ZozIdent BCIARKA
        | VyrazPr BCIARKA
        | RETURN BCIARKA
        | RETURN Vyraz BCIARKA
        | WHILE LOZAT Vyraz POZAT Prikaz
        | IF LOZAT Vyraz POZAT Prikaz ELSE Prikaz
        | IF LOZAT Vyraz POZAT Prikaz
        | LZZAT Prikazy PZZAT
        ;
ZozIdent : ID
        | ZozIdent CIARKA ID
        ;
Vyraz   : Primarny
        | Vyraz PLUS Vyraz
        | Vyraz MINUS Vyraz
        | Vyraz KRAT Vyraz
        | Vyraz DEL Vyraz
        | Vyraz LT Vyraz
        | Vyraz LE Vyraz
        | Vyraz GT Vyraz
        | Vyraz GE Vyraz
        | Vyraz EQ Vyraz
        | Vyraz NE Vyraz
        | Vyraz OR Vyraz
        | Vyraz AND Vyraz
        | PLUS Vyraz %prec UNARNY
        | MINUS Vyraz %prec UNARNY
        ;
VyrazPr :  ID ROVNE Vyraz
        |  DCPrirad
        |  MetVolanie
        |  LOZAT VOID POZAT MetVolanie
        ;
Primarny : ID
        | THIS
        | NovyObjekt
        | MetVolanie
        | DCPristup
        | NULL_LITERAL
        | INT_LITERAL
        | DBL_LITERAL
        | STR_LITERAL
        | TRUE_LITERAL
        | FALSE_LITERAL
        | LOZAT  Vyraz  POZAT
        | TRIEDA LOZAT Vyraz POZAT
        | INT LOZAT Vyraz POZAT
        | DOUBLE LOZAT Vyraz POZAT
        | LONG LOZAT Vyraz POZAT
        ;
NovyObjekt : NEW  TRIEDA  LOZAT  NepArgumenty  POZAT
        ;
MetVolanie : TRIEDA BODKA ID LOZAT NepArgumenty POZAT
        | SUPER LOZAT NepArgumenty POZAT
        | Primarny BODKA ID  LOZAT  NepArgumenty POZAT
        ;
DCPristup : TRIEDA BODKA ID
        | Primarny BODKA ID
        ;
DCPrirad : TRIEDA BODKA ID  ROVNE Vyraz
        | Primarny BODKA ID  ROVNE Vyraz
        ;
NepArgumenty : /* e */
        | Argumenty
        ;
Argumenty : Vyraz
        | Argumenty  CIARKA  Vyraz
        ;
%%

static int pocetChyb = 0;

void yyerror(char *s)
{
    fprintf(stdout, "CHYBA: %s [%s:%d]\n", s, yytext, riadok);
    pocetChyb++;
}

void xxerror(char *s1, char *s2)
{
    fprintf(stdout, "CHYBA: %s %s [%d]\n", s1, s2, riadok);
    pocetChyb++;
}

/* Nazov vstupneho suboru */
static char src[20];

int main(int argc, char *argv[])
{
    extern FILE *yyin;
    int i;
    if (argc > 1) {
        yyin = fopen(argv[1], "r");
        strcpy(src, argv[1]);
        if (yyin == NULL) {
            fprintf(stderr, "Nemozno otvorit zdrojovy subor, citanie zo stand. vstupu\n");
            yyin = stdin;
            strcpy(src, "stdin");
        }
    }
    (void) yyparse();
    fprintf(stderr, "POCET CHYB: %d\n", pocetChyb);
    fprintf(stderr, "Stlacte klaves\n");
    getchar();
    return 0;
}
